unit RGS_BMChooser;

{ TRgsBMChooser Version 1.0
  Author : Tnis Argus
  Status: Freeware
  eMail : Tonis.Argus@mail.ee

  This component enables You record all bookmarks in a collection,
  assign each bookmark a name and then view bookmarks by name
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB,
  dsgnintf, extctrls, stdctrls, RGS_DBInspector;

type
  TRGSBMChooser = class;
  TRGSBMInspector = class(TCustomListBox)
  private
    HalfWay: Integer;
    SplitDrag : Boolean;
    EditRect : TRect;
    FAutoCalcSep: Boolean;
    FActive: Boolean;
    FRGSBMChooser : TRGSBMChooser;
    procedure SetAutoCalcSep(Value: Boolean);
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton;  Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CheckFields(aDataset : TDataset);
  published
    property AutoCalcSep: Boolean read FAutoCalcSep write SetAutoCalcSep;
    property Align;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ItemHeight;
    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style default lbOwnerDrawFixed;
    property TabOrder;
    property Visible;
    property Width;
  end;


  TRGSBMOnChange = procedure(Sender: TObject; Dataset : TDataset) of object;
  TRGSBMOnNewBookmark = procedure(Sender: TObject; Dataset : TDataset; Var BookmarkName : string) of object;

  TRGSBMChooserItem = class(TCollectionItem)
  private
    FDataset: TDataset;
    FBookmark : TBookmark;
    FBookmarkName : String;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
 published
    property Dataset: TDataset read FDataset write FDataset;
    property Bookmark: TBookmark read FBookmark write FBookmark;
    property BookmarkName : String read FBookmarkName write FBookmarkName;
  end;

  TRGSBMChooserCollection = class(TCollection)
  protected
    FOwner: TPersistent;
    function GetOwner:TPersistent; override;
    function GetItem(Index: Integer): TRGSBMChooserItem;
    procedure SetItem(Index: Integer; Value: TRGSBMChooserItem);
  public
    property Items[Index: Integer]: TRGSBMChooserItem read GetItem write SetItem; default;
  end;

  TRGSBMChooser = class(TCustomPanel)
  private
    FRGSBMOnNewBookmark: TRGSBMOnNewBookmark;
    FBMCollection: TRGSBMChooserCollection;
    FRGSBMInspector: TRGSBMInspector;
    FTopPanel: TPanel;
    FComboBox: TComboBox;
    FTextMemo : String;
    FTextImage : String;
    procedure TopPanelResize(Sender: TObject);
    procedure ComboBoxChange(Sender: TObject);
  protected
    procedure SetCollection(const Value: TRGSBMChooserCollection);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure AddBookmark(aDataset : TDataset);
  published
    property TextMemo : String read FTextMemo write FTextMemo;
    property TextImage : string read FTextImage write FTextImage;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property Enabled;
    property Visible;
    property TabOrder;
    property OnResize;
    property OnNewBookmark : TRGSBMOnNewBookmark
    read FRGSBMOnNewBookmark write FRGSBMOnNewBookmark;
//    property OnDatasourceChange : TRGSBMChooserChangeEvent
//             read FOnDatasourceChange write FOnDatasourceChange;
  end;

procedure Register;

implementation


{ TRGSBMInspector }

procedure TRGSBMInspector.CreateParams;
begin
  inherited CreateParams(Params);
  with Params do ExStyle := ExStyle or WS_EX_TOOLWINDOW;
end;

constructor TRGSBMInspector.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoCalcSep := True;
  ItemHeight := 18;
  Color := clBtnFace;
  Style := lbOwnerDrawFixed;
  Width := 160;
end;

destructor TRGSBMInspector.Destroy;
begin
  inherited Destroy;
end;

procedure TRGSBMInspector.SetAutoCalcSep(Value: Boolean);
begin
  if FAutoCalcSep <> Value then
    FAutoCalcSep := Value;
end;

procedure TRGSBMInspector.CheckFields(aDataset : TDataset);
var
  X : Integer;
  Field : TField;
begin
  Clear;
  try
    HalfWay := 0;
    if not FAutoCalcSep then
      HalfWay := Width div 2;
    for X := 0 to aDataSet.FieldCount-1 do
      Begin
      Field := aDataSet.Fields[X];
      if(Field.Visible) then
        Begin
          if FAutoCalcSep then
            if Canvas.TextWidth(Field.DisplayLabel)+10 > HalfWay then
              HalfWay := Canvas.TextWidth(Field.DisplayLabel)+10;
          Items.AddObject('', Field);
        end;
      end;
  finally
  end;
end;

procedure TRGSBMInspector.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  EdString, RString: string;
  F: TField;
  S: String;
  SRect: TRect;
begin
    F := TField(Items.Objects[Index]);
    If F = nil Then
      exit;
    HideCaret(Handle);
    S := '';
//    for V := 0 to PEditData.Depth do
      S := S + '  ';
    { draw the item }
    with Canvas do
    begin
      Brush.Color := Self.Color;  {Won't use Windows Selected color.}
      SRect := Rect;
      If odSelected in state then
        SRect.Right := HalfWay;
      FillRect(SRect);
      if (odSelected in State) then
        Pen.Color := clBtnHighLight
      else {for 3D look}
        Pen.Color := clBtnShadow;
      Pen.Mode := pmCopy;
      Pen.Width := 1;
      Pen.Style := psSolid;
      MoveTo(rect.Left, Rect.Bottom-1);
      LineTo(rect.Right, Rect.Bottom-1);
      if (odSelected in State) then
      begin
        Pen.Color := clBtnHighLight;
        MoveTo(HalfWay + 2, Rect.Top+1);
        LineTo(HalfWay + 2, Rect.Bottom);

        Pen.Color := clBtnShadow;
        MoveTo(HalfWay+1, Rect.Top+2);
        LineTo(HalfWay+1, Rect.Bottom);

        MoveTo(Rect.Right, Rect.Top);
        LineTo(Rect.Left, Rect.Top);
        LineTo(Rect.Left, Rect.Bottom);

        Pen.Color := clBlack;
        MoveTo(Rect.Right - 1, Rect.Top + 1);
        LineTo(Rect.Left + 1, Rect.Top + 1);
        LineTo(Rect.Left + 1, Rect.Bottom - 1);
      end else
      begin
        Pen.Color := clBtnShadow;
        MoveTo(HalfWay, Rect.Top); //Top+1
        LineTo(HalfWay, Rect.Bottom);

        Pen.Color := clBtnHighLight;
        MoveTo(HalfWay + 1, Rect.Top); // Top+1
        LineTo(HalfWay + 1, Rect.Bottom);
      end;

      SRect.Left := Rect.Left;
      SRect.Right := HalfWay;
      SRect.Top := Rect.Top+1;
      SRect.Bottom := Rect.Bottom-1;
      Font.Color := clBlack;

      If (odSelected in State) then
      Begin
        Inc(SRect.Left);
        Inc(SRect.Top);
        Inc(SRect.Right);
      End;

      setBkMode(Canvas.Handle, Transparent);

      TextRect(SRect, SRect.Left, SRect.Top, S + F.DisplayLabel);

      Pen.Color := clBtnFace;
      Font.Color := clNavy;

      If F.DataType in [ftTypedBinary, ftBlob, ftMemo, ftFmtMemo] Then
        RString := FRGSBMChooser.FTextMemo
      else If F.DataType = ftGraphic Then
        RString := FRGSBMChooser.FTextImage
      else
        RString := F.DisplayText;  //F.AsString
      Canvas.Font.Color := clNavy; // Text color on the right side
      SRect := ItemRect(ItemIndex);
      if (odSelected in State) then
      With Canvas do
      Begin
        EditRect := ItemRect(ItemIndex);
        EditRect.Left := HalfWay + 2;
        Brush.Color := clWhite;
        Inc(EditRect.Top, 2);
        Fillrect(EditRect);
        setBkColor(Canvas.Handle, GetSysColor(color_HighLight));
        setBkMode(Canvas.Handle, Opaque);
        Font.Color := clHighlightText;
        Canvas.Brush.Color := clHighlight;
        EdString := RString;
        while (TextWidth(EdString) > (EditRect.Right - EditRect.Left)) and
          (EdString <> '')  do
          Delete(EdString, 1, 1);
        TextOut(EditRect.Left + 2, EditRect.Top+1, edString);
        DestroyCaret;
        CreateCaret(Handle, 0, 1, (EditRect.Bottom - Editrect.Top) - 2);
        SetCaretPos(EditRect.Left + TextWidth(EdString), EditRect.Top + 1);
        ShowCaret(Handle);
      end else
      begin
        Canvas.TextOut(HalfWay + 3, Rect.Top+1, RString);
      End;
      if (odFocused in State) and (odSelected in State) then
        Canvas.DrawFocusRect(Rect);
    end;
end;

procedure TRGSBMInspector.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  If Abs(x - Halfway) <= 3 then
    SplitDrag := True;
end;

procedure TRGSBMInspector.MouseMove(Shift: TShiftState; X, Y: Integer);
Var
  TempR : TRect;
begin
  If SplitDrag then
  Begin
    If X > ClientWidth - 20 then
      X := ClientWidth - 20;
    If X < 20 then
      X := 20;
    If Halfway < x then
    Begin
      TempR.Left := Halfway;
      TempR.Right := ClientWidth;
      TempR.Top := 0;
      TempR.Bottom := ClientHeight;
    End Else
    Begin
      TempR.Left := X;
      TempR.Right := ClientWidth;
      TempR.Top := 0;
      TempR.Bottom := ClientHeight;
    End;
    HalfWay := x;
    InvalidateRect(Handle, @tempR, false);
    InvalidateRect(Handle, @EditRect, False);
    PostMessage(Handle, WM_Paint, 0, 0);
    Exit;
  End;
  If Abs(x - HalfWay) <= 3 then
    Cursor := crHSplit
  Else
  If (X >= EditRect.left) and (X <= EditRect.Right) and
     (y >= EditRect.Top) and (Y <= EditRect.Bottom) then
    Cursor := crIBeam else Cursor := crDefault;
End;

procedure TRGSBMInspector.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  SplitDrag := False;
end;

procedure TRGSBMInspector.KeyDown(var Key: Word; Shift: TShiftState);
begin
  If (Key = VK_RETURN) or (Key = VK_SPACE) then
  begin
    Click;
    Key := 0;
    Exit;
  end;
end;

{TRGSBMChooserItem}

constructor TRGSBMChooserItem.Create(Collection: TCollection);
begin
   inherited Create(Collection);
end;

destructor TRGSBMChooserItem.Destroy;
begin
   inherited;
end;

procedure TRGSBMChooserItem.Assign(Source: TPersistent);
begin
   if Source is TRGSBMChooserItem then
      begin
        Dataset := TRGSBMChooserItem(Source).Dataset;
        Bookmark := TRGSBMChooserItem(Source).Bookmark;
        BookmarkName := TRGSBMChooserItem(Source).BookmarkName;
      end
   else
     inherited Assign(Source);
end;

function TRGSBMChooserCollection.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TRGSBMChooserCollection.GetItem(Index: Integer): TRGSBMChooserItem;
begin
   Result := TRGSBMChooserItem(inherited GetItem(Index));
end;

procedure TRGSBMChooserCollection.SetItem(Index: Integer; Value: TRGSBMChooserItem);
begin
   inherited SetItem(Index, Value);
end;

constructor TRGSBMChooser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 200;
  Width := 160;
  FBMCollection := TRGSBMChooserCollection.Create(TRGSBMChooserItem);
  FBMCollection.FOwner := Self;

  FTopPanel:= TPanel.Create(Self);
  FTopPanel.Parent := Self;
  FTopPanel.Align := alTop;
  FTopPanel.Height := 25;
  FTopPanel.OnResize := TopPanelResize;

  FComboBox:= TComboBox.Create(Self);
  FComboBox.Parent := FTopPanel;
  FComboBox.Top := 0;
  FComboBox.Left := 0;
  FComboBox.Width := FTopPanel.Width-3;
  FComboBox.OnChange := ComboBoxChange;

  FRGSBMInspector:= TRGSBMInspector.Create(Self);
  FRGSBMInspector.Parent := Self;
  FRGSBMInspector.Align := alClient;
  FRGSBMInspector.FRGSBMChooser := self;

end;

destructor TRGSBMChooser.Destroy;
begin
   FBMCollection.Free;
   inherited Destroy;
end;

Procedure TRGSBMChooser.AddBookmark(aDataset : TDataset);
Var
  aBookmarkName : string;
  aItem : TRGSBMChooserItem;
Begin
  aBookmarkName := '';
  if Assigned(FRGSBMOnNewBookmark) then
    FRGSBMOnNewBookmark(Self, aDataset, aBookmarkName);
  aItem := FBMCollection.Add As TRGSBMChooserItem;
  With aItem do
    Begin
      Bookmark := aDataset.GetBookmark;
      BookmarkName := aBookmarkName;
      Dataset := aDataset;
    end;
  FComboBox.Items.Add(aBookmarkName);
end;

procedure TRGSBMChooser.TopPanelResize(Sender: TObject);
begin
  FComboBox.Width := FTopPanel.ClientWidth;
end;

procedure TRGSBMChooser.ComboBoxChange(Sender: TObject);
begin
  try
    With FBMCollection.Items[FComboBox.ItemIndex] do
      Begin
        If Dataset.BookmarkValid(Bookmark) Then
          Begin
            Dataset.GotoBookmark(Bookmark);
            FRGSBMInspector.CheckFields(Dataset);
          end;
      end;
  except
  end;
{  FRGSBMInspector.DataSource :=  FDSCollection.Items[FComboBox.ItemIndex].Datasource;
  if Assigned(FOnDatasourceChange) then
    FOnDatasourceChange(Self, FRGSBMInspector.DataSource);
}
end;

procedure TRGSBMChooser.SetCollection(const Value: TRGSBMChooserCollection);
begin
   FBMCollection.Assign(Value);
end;

procedure TRGSBMChooser.Loaded;
Var
  i : Integer;
begin
  inherited;
  For i := 0 To FBMCollection.Count-1 do
    FComboBox.Items.Add(FBMCollection.Items[i].BookmarkName);
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TRGSBMChooser]);
end;

end.
