unit RGS_StringStore;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB, dsgnintf;

type
  TInfoItem = class(TCollectionItem)
  private
    FName: String;
    FValue: String;
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
 published
    property Name: string read FName write FName;
    property Value: String read FValue write FValue;
  end;

  TInfoCollection = class(TCollection)
  protected
    FOwner: TPersistent;
    function GetOwner:TPersistent; override;
    function GetItem(Index: Integer): TInfoItem;
    procedure SetItem(Index: Integer; Value: TInfoItem);
  public
    property Items[Index: Integer]: TInfoItem read GetItem write SetItem; default;
  end;

  TRGSStringStore = class(TComponent)
  private
    FInfoItems: TInfoCollection;
  protected
    procedure SetCollection(const Value: TInfoCollection);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Function GetString(aName, Default : String) : String;
    procedure SetString(aName, aValue : String);
    Function GetInteger(aName : String; Default : Integer) : Integer;
    Procedure SetInteger(aName : String; aValue: Integer);
  published
    property Items: TInfoCollection read FInfoItems write SetCollection;
  end;

  TRGSStringStoreEditor = class(TComponentEditor)
    function GetVerbCount: integer; override;
    function GetVerb(Index: integer): string; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

procedure Register;

implementation

constructor TInfoItem.Create(Collection: TCollection);
begin
   inherited Create(Collection);
end;

destructor TInfoItem.Destroy;
begin
   inherited;
end;

procedure TInfoItem.Assign(Source: TPersistent);
begin
   if Source is TInfoItem then
      begin
        Name := TInfoItem(Source).Name;
        Value := TInfoItem(Source).Value;
      end
   else
     inherited Assign(Source);
end;

function TInfoItem.GetDisplayName: string;
begin
   Result := Name;
   if Result = '' then
     Result := inherited GetDisplayName;
end;

function TInfoCollection.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TInfoCollection.GetItem(Index: Integer): TInfoItem;
begin
   Result := TInfoItem(inherited GetItem(Index));
end;

procedure TInfoCollection.SetItem(Index: Integer; Value: TInfoItem);
begin
   inherited SetItem(Index, Value);
end;

constructor TRGSStringStore.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FInfoItems := TInfoCollection.Create(TInfoItem);
   FInfoItems.FOwner := Self;
end;

destructor TRGSStringStore.Destroy;
begin
   FInfoItems.Free;
   inherited Destroy;
end;

Function TRGSStringStore.GetString(aName, Default : String) : String;
Var
  i : Integer;
Begin
  result := Default;
  For i := 0 to FInfoItems.Count-1 do
    If FInfoItems[i].Name = aName Then
      begin
        result := FInfoItems[i].Value;
        Break;
      end;
end;

Procedure TRGSStringStore.SetString(aName, aValue : String);
Var
  i : Integer;
  ok : Boolean;
  aInfoItem : TInfoItem;
Begin
  ok := False;
  For i := 0 to FInfoItems.Count-1 do
    If FInfoItems[i].Name = aName Then
      begin
        FInfoItems[i].Value := aValue;
        ok := True;
        Break;
      end;
  If not Ok Then
    Begin
      aInfoItem := TInfoItem.Create(FInfoItems);
      aInfoItem.Name := aName;
      aInfoItem.Value := aValue;
    end;
end;

Function TRGSStringStore.GetInteger(aName : String; Default : Integer) : Integer;
Var
  i : Integer;
Begin
  result := Default;
  For i := 0 to FInfoItems.Count-1 do
    If FInfoItems[i].Name = aName Then
      begin
        try
          result := StrToInt(FInfoItems[i].Value);
        except
          result := Default;
        end;
        Break;
      end;
end;

Procedure TRGSStringStore.SetInteger(aName : String; aValue: Integer);
Var
  i : Integer;
  ok : Boolean;
  aInfoItem : TInfoItem;
Begin
  ok := False;
  For i := 0 to FInfoItems.Count-1 do
    If FInfoItems[i].Name = aName Then
      begin
        FInfoItems[i].Value := IntToStr(aValue);
        ok := True;
        Break;
      end;
  If not Ok Then
    Begin
      aInfoItem := TInfoItem.Create(FInfoItems);
      aInfoItem.Name := aName;
      aInfoItem.Value := IntToStr(aValue);
    end;
end;

procedure TRGSStringStore.SetCollection(const Value: TInfoCollection);
begin
   FInfoItems.Assign(Value);
end;

procedure TRGSStringStore.Loaded;
begin
   inherited;
end;

function TRGSStringStoreEditor.GetVerbCount: integer;
begin
   Result := 1;
end;

function TRGSStringStoreEditor.GetVerb(Index: integer): string;
begin
   Result := 'Saved strings...';
end;

procedure TRGSStringStoreEditor.ExecuteVerb(Index: integer);
begin
  Designer.Modified;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TRGSStringStore]);
  RegisterComponentEditor(TRGSStringStore, TRGSStringStoreEditor);
end;

end.